import re

from datetime     import datetime, timedelta
from ..core.tools import log
from .analyzer    import Analyzer
from ..downloader.youtube import (
	download_xml,
	download_html,
	download_show_more,
	download_html_playlist
)


class Youtube_Analyzer(Analyzer):
	SITE='[youtube]'
	URL_MATCH=r'(?:https://|)(?:www\.|)youtube\.com/(?P<type>channel/|user/|playlist\?list=)(?P<ID>[a-zA-Z0-9_-]*)'
	RE_CHANNEL=r'UC[A-Za-z0-9_-]{22}'
	RE_PLAYLIST=r'PL[A-Za-z0-9_-]{32}'

	def __init__(self, url_id='', method='0', file=None, prog=None):
		self.id = self.extract_id(url_id)
		self.method = method
		self.type = self._type_id(url_id)  # True --> chanel False -- > Playlist
		# Init info videos
		self.channel = ''
		self.date = ''
		self.title = ''
		self.url = ''
		self.url_channel = ''
		self.url_img = ''
		self.view = ''
		# Function
		self.prog = prog
		self.file = file

	def add_sub(self, url):
		sub = self.extract_id(url)
		tid = self._type_id(sub)
		data = download_xml(sub, type_id=tid, split=False)
		if data is None:
			log.Error("The channel/playlist can't be add. It could be delete.")
			return None
		match = re.findall(r'<(?:title|name)>(.+?)</(?:title|name)>', data)
		if match != []:
			return sub + '\t' + match[0]
		log.Warning("The channel/playlist can't be add. It could be delete.")
		return None

	def real_analyzer(self):
		"""Recover all the videos of a channel or a playlist
		and add the informations in $HOME/.cache/youtube_sm/data/."""
		linfo = self._download_page()
		if linfo is False or linfo is None:
			return
		if self.method == '0':
			for i in linfo:
				self.info_rss(i)
				self.write()
		elif self.method == '1':
			if self.type:  # Channel
				channel = re.findall('<title>(.*)', linfo[0])
				if channel == []:
					log.Error('Title not found ({})'.format(self.id))
				else:
					self.channel = channel[0]
				del linfo[0]
			for i in linfo:
				try:
					self.info_html(i)
					self.write()
				except:
					log.Error('Error during the retrieve of the info ({})'.format(self.id))
		elif self.method == '2':
			if self.type:
				channel = re.findall('<title>(.*)', linfo[0])[0]
				if channel == []:
					log.Error('Title not found ({})'.format(self.id))
				else:
					self.channel = channel[0]
				del linfo[0]
				for i in linfo:
					try:
						self.info_html(i)
						self.write()
					except:
						log.Error('Error during the retrieve of the info ({})'.format(self.id))
				nexts = re.findall(r'data-uix-load-more-href="([^"]*)"', linfo[-1])
				if nexts == []:
					return
				self.next = nexts[0]
			self.show_more()

	def _download_page(self):
		""" To download a page with the id of a channel/Playlist """
		if self.method == '0':
			return download_xml(self.id, self.type)
		elif self.method == '1':
			return download_html(self.id, self.type)
		elif self.method == '2':
			if self.type:
				return download_html(self.id, self.type)
			else:
				linfo, self.next, self.len_play = download_html_playlist(self.id)
				return linfo
		else:
			return None

	def write(self):
		"""Write the information in a file"""
		return self.file.add(
			uploader=self.channel,
			date=self.date,
			title=self.title,
			url='https://www.youtube.com/watch?v='+self.url,
			url_uploader=self._complet_url_channel(self.url_channel),
			image='https://i.ytimg.com/vi/{}/mqdefault.jpg'.format(self.url),
			view=self.view,
		)

	def show_more(self):
		""" The continuation of real_analyzer for the mode 'ultra-html'
		This function recover and write the information recover in
		in the link in the button 'load more'"""
		data = ''
		if self.type:
			while self.next is not None:
				data, self.next = download_show_more(self.next, True)
				if data is None:
					return
				for i in data:
					try:
						self.info_show_more(i)
						self.write()
					except:
						log.Error('Error during the retrieve of the info ({})'.format(self.id))
				if self.prog is not None:
					self.prog.add()
		else:
			nb = 0
			while nb < self.len_play:
				data, self.next = download_show_more(self.next, False)
				if data is None:
					return
				for i in data:
					index = re.findall(r'tch\?v=.*;index=([0-9]+?)', i)
					if index != [] and nb < int(index[0]):
						nb += 1
						try:
							self.info_html(i)
							self.write()
						except:
							log.Error('Error during the retrieve of the info ({})'.format(self.id))
				if self.prog is not None:
					self.prog.add()
		print()

	def info_show_more(self, i):
		"""Recover the informations for the mode 'ultra-html'"""
		self.url = re.findall(r'href="\\/watch\?v=(.{11})"', i)[0]
		self.url_channel = self.id
		self.title = re.findall(r'ltr"\ title="(.+?)"', i)[0]
		date = re.findall(r'/li\\u003e\\u003cli\\u003e(.*)\\u003c\\/li', i)[0]
		self.date = self.date_convert(date)
		self.view = re.findall(r'class="yt-lockup-meta-info"\\u003e\\u003cli\\u003e(.+?) views', i)[0].replace(',', '')

	def info_html(self, i):
		"""Recover the informations of the html page"""
		if self.type:  # Channel
			self.url         = re.findall(r'href="/watch\?v=(.{11})"', i)[0]
			self.url_channel = self.id
			self.title       = re.findall(r'dir="ltr" title="(.+?)"', i)[0]
			date             = re.findall(r'</li><li>(.+?)</li>', i)[0]
			self.date        = self.date_convert(date)
			self.view        = re.findall(r'class="yt-lockup-meta-info"><li>(.+?)\ views', i)[0].replace(',', '')
		else:  # Playlist
			self.url         = re.findall(r'data-video-id="(.{11})"', i)[0]
			self.title       = re.findall(r'data-video-title="(.+?)"', i)[0]
			self.channel     = re.findall(r'data-video-username="(.+?)"', i)[0]
			self.url_channel = self.id
			self.date        = datetime.now()

	def info_rss(self, i):
		"""Recover the informations of a rss page"""
		self.url         = re.findall(r'<yt:videoId>(.{11})</yt:videoId>', i)[0]
		self.url_channel = re.findall(r'<yt:channelId>(.*)</yt:channelId>', i)[0]
		self.title       = re.findall(r'<media:title>(.*)</media:title>', i)[0]
		self.channel     = re.findall(r'<name>(.*)</name>', i)[0]
		date             = re.findall(r'<published>(.*)\+', i)[0]
		self.date        = datetime.strptime(date, '%Y-%m-%dT%H:%M:%S')
		self.view        = re.findall(r'views="(.+?)"', i)[0].replace(',', '')

	def _type_id(self, id):
		"""True = Channel; False = Playlist"""
		if re.match(self.RE_CHANNEL, id):
			return True
		elif re.match(self.RE_PLAYLIST, id):
			return False
		else:
			return True

	def _complet_url_channel(self, url_channel):
		if url_channel[:2] == 'UC':
			url_channel = 'https://youtube.com/channel/' + url_channel
		elif url_channel[:8] == 'results?':
			url_channel = 'https://youtube.com/' + url_channel
		else:
			url_channel = 'https://youtube.com/user/' + url_channel
		return url_channel

	def date_convert(self, date):
		"""
		Convert "2 day ago" -> datetime format
		"""
		value, unit, _ = date.split()
		if unit in ['month', 'months']:
			unit = 'days'
			value = int(value) * 31
		elif unit in ['year', 'years']:
			unit = 'days'
			value = int(value) * 365
		elif unit in ['minute', 'hour', 'day', 'week']:
			unit += 's'
		delta = timedelta(**{unit: int(value)})
		date = datetime.now() - delta
		return date

	def old(self, url, lcl):
		linfo = download_xml(url)
		if linfo is False:
			print('[channel dead]', url)
		elif linfo is None:
			print('[  no video  ]', url)
		else:
			tps_vd = re.findall(r'<published>([0-9-]{10})T.+?</published>', linfo[0])[0]
			if lcl > int(tps_vd.replace('-', '')):
				print('[ {} ] {}'.format(tps_vd.replace('-', '/'), re.findall('<name>(.+?)</name>', linfo[0])[0]))

	def dead(self, url):
		""" Print the dead channel """
		linfo = download_xml(url)
		if linfo is False:
			print('[channel dead]\t', url)
		elif linfo is None:
			print('[  no video  ]\t', url)



import re
import sys
from os.path import exists
from os import makedirs
from shutil import rmtree

from ..analyzer.imports import (
	return_Analyzer,
	UrlToAnalyzer,
)
from .tools import (
	log)


def write_list(list_subs, path):
	open(path + 'sub.swy', 'w', encoding='utf8').write('[v][2.0]\n')
	file = open(path + 'sub.swy', 'a', encoding='utf8')
	for site in list_subs:
		file.write('[site]' + site + '\n')
		for i in list_subs[site]:
			if i is not None:
				file.write(i + '\n')


def generate_swy(sub_file, path=''):
	"""
	Add sub in sub.swy
	"""
	if exists(sub_file):
		data = open(sub_file, 'r', encoding="utf8").read()
	else:
		exit('[!] File not found (' + sub_file + ')')
	liste = data.split('<outline')
	del liste[:2]
	if not exists(path + 'sub.swy'):
		open(path + 'sub.swy', 'a', encoding='utf8').write('[v][2.0]\n[site][youtube]\n')
		data_sub = {'[youtube]': []}
	else:
		data_sub = swy(path, 2)
	for i in liste:
		channel = re.findall(r'title="(.+?)"', i)[0]
		id_chnl = re.findall(r'channel_id=(.+?)"', i)[0]
		if id_chnl+'\t'+channel not in data_sub['[youtube]']:
			data_sub['[youtube]'].append(id_chnl + '\t' + channel)
	write_list(data_sub, path)


def add_suburl(url, path=''):
	list_subs = swy(path, 2)
	analyzer = UrlToAnalyzer(url)
	if analyzer is None:
		log.Error('The url {} is not support'.format(url))
		return
	analyzer = analyzer()
	if analyzer is None:
		log.Error('The url {} is not support'.format(url))
		return
	data = analyzer.add_sub(url)
	if data is None:
		log.Error('Failed to add ', url)
	else:
		log.RInfo('Add: ', data)
	if data is not None:
		if analyzer.SITE in list_subs:
			list_subs[analyzer.SITE].append(data)
		else:
			list_subs[analyzer.SITE] = [data]
	write_list(list_subs, path)


def add_sub(subs, path=''):
	"""
	Add a list of subs in sub.swy
	"""
	list_subs = swy(path, 2)
	for site in subs:
		analyzer = return_Analyzer(site)
		if analyzer is None:
			log.Error('The site {} is not support'.format(site))
			continue
		analyzer = analyzer()
		if analyzer is None:
			log.Error('The site {} is not support'.format(site))
			continue
		for sub in subs[site]:
			data = analyzer.add_sub(sub)
			if data is not None:
				if site in list_subs:
					list_subs[site].append(data)
				else:
					list_subs[site] = [data]
	write_list(list_subs, path)


def swy(path, mode=0):
	"""
	Return a list of sub wich are in sub.swy
	mode :	0 --> return a list with only the id
			1 --> return a dict with the channel and the id
			2 --> return a list wich is not .split('\t')
	"""
	log.Info('Start read swy')
	if not exists(path + 'sub.swy'):
		try:
			open(path + 'sub.swy', 'w', encoding='utf8')
		except:
			log.Error('You didn\'t initialize')
	urls = dict()
	data_sub = open(path + 'sub.swy', 'rb').read().decode("utf8").split('[site]')
	del data_sub[0]
	if mode == 0 or mode == 2:
		for i in data_sub:
			subs = i.split('\n')
			siteid = subs[0].rstrip()
			urls[siteid] = []
			for y in range(1, len(subs)):
				if subs[y] == '':
					continue
				if mode == 0:
					urls[siteid].append(subs[y].split('\t')[0])
				else:
					urls[siteid].append(subs[y])
	elif mode == 1:
		for i in data_sub:
			subs = i.split('\n')
			siteid = subs[0].rstrip()
			urls[siteid] = dict()
			for y in range(1, len(subs)):
				if subs[y] == '':
					continue
				urls[siteid][subs[y].split('\t')[0]] = subs[y].split('\t')[1]
	for site in urls:
		log.Info("{} subs for {}".format(len(urls[site]), site))
	return urls


def init_swy(path, arg):
	if exists(path):
		rmtree(path)
	makedirs(path + 'data/')
	log.Info('Data Folder Create')
	if len(sys.argv) != arg + 1:
		add_file = sys.argv[arg + 1]
		if not exists(add_file):
			exit('[!] File Not Found (' + add_file + ')')
		if add_file[len(add_file) - 4:] == '.swy':
			with open(add_file, 'rb') as file, open(path + 'sub.swy', 'a', encoding='utf8') as sub_file:
				nb_line = 1
				while True:
					line = file.readline().decode('utf8')
					if '\t' in line or '[site]' in line or '[v]' in line:
						sub_file.write(line)
					elif line == '':
						break
					else:
						log.Error('No tabs in line ' + str(nb_line))
		elif add_file[len(add_file) - 5:] == '.json':
			subs = dict()
			data = open(add_file, 'rb').read().decode('utf8')
			subs['[youtube]'] = re.findall(r'channel/(.{24})', data)
			add = re.findall(r'youtube.com/playlist\?list=(.{34})', data)
			subs['[youtube]'] = [*subs['[youtube]'], *add]
			add_sub(subs, path)
		else:
			generate_swy(add_file, path=path)
	else:
		if exists('subscription_manager'):
			generate_swy('subscription_manager', path=path)
		else:
			log.Error('File Not Found (subscription_manager)')
			exit()
	log.Info('Done')
	exit(0)



import re

from ..core.tools import log
from ..core.sock  import (
	download_http,
	download_https
)


def download_page(url_id, type_id=True, split=True, method='0'):
	if method == '0':
		return download_xml(url_id, type_id, split)
	elif method == '1':
		return download_html(url_id, type_id, split)
	else:
		return None


def download_xml(url_id, type_id=True, split=True):
	"""
	Return a list of informations of each video with the
	RSS youtube page
	"""
	if type_id:  # Channel
		url = '/feeds/videos.xml?channel_id=' + url_id
	else:  # Playlist
		url = '/feeds/videos.xml?playlist_id=' + url_id
	site = download_http(url)
	if site.status != '200':
		return None
	data = site.body
	if split:
		linfo = data.split("<entry>")
		del linfo[0]
		if linfo == []:
			log.Warning('No videos ({})'.format(url_id))
			return None
		return linfo
	else:
		return data


def download_html(url_id, type_id=True, split=True):
	"""
	Return a list of informations of each video with the
	current html page
	"""
	if type_id:
		url = '/channel/' + url_id + '/videos'
	else:
		url = '/playlist?list=' + url_id
		site = download_https(url)
		if site.status != '200':
			return None
		data = site.body
		url = data.split('<a href="/watch?v')[1].split('"')[0]
		url = '/watch?v' + url
	site = download_https(url)
	if site.status != '200':
		return None
	data = site.body
	if split:
		if type_id:  # Channel
			linfo = data.split('<div class="yt-lockup-content">')
			if len(linfo) <= 1:
				log.Warning('No videos ({})'.format(url_id))
				return None
		else:  # Playlist
			linfo = data.split('<li class="yt-uix-scroller-scroll-unit  vve-check"')
			del linfo[0]
			if linfo == []:
				log.Warning('No videos ({})'.format(url_id))
				return None
		return linfo
	else:
		return data


def download_html_playlist(url_id, split=True):
	"""
	Download the html page of a playlist and
	return the data, the next link to download and the size of the playlist
	"""
	data = download_html(url_id, False, False)
	try:
		len_play = int(re.findall(r'<span id="playlist-length">(.+?)videos</span>', data)[0])
	except:
		log.Warning("No video in this page ({})".format(url_id))
		return None, None, None
	linfo = data.split('<li class="yt-uix-scroller-scroll-unit  vve-check"')
	del linfo[0]
	if linfo == []:
		log.Warning("No video in this page ({})".format(url_id))
		return None, None, None
	try:
		next_link = '/watch?v=' + re.findall(r'<a href="/watch\?v=(.+?)"', linfo[-1])[0]
	except:
		next_link = None
	return linfo, next_link, len_play


def download_show_more(url, type_id=True):
	"""
	Download the page for the method 'ultra-html'
	and return the data and the next link to download
	"""
	if type_id:
		site = download_https(url)
		data = data.replace('\\n', '\n').replace('\\"', '"')
		if site.status != '200':
			return None
		data = site.body
		try:
			next_link = data.split('data-uix-load-more-href="\\')[1].split('"')[0]
		except:
			next_link = None
		data = data.split('yt-lockup-content')
		del data[0]
		return data, next_link
	else:
		site = download_https(url)
		if site.status != '200':
			return None
		data = site.body
		linfo = data.split('<li class="yt-uix-scroller-scroll-unit  vve-check"')
		del linfo[0]
		try:
			next_link = '/watch?v=' + re.findall(r'<a href="/watch\?v=(.+?)"', linfo[-1])[0]
		except:
			return None, None
		return linfo, next_link



import re
from datetime import datetime

from .analyzer	import Analyzer
from ..downloader.reddit import download_rss_reddit

class Reddit_Analyzer(Analyzer):
	SITE='[reddit]'
	URL_MATCH=r'(?:https://|)(?:www\.|)reddit\.com/(?P<ID>(?:r|user)/[A-z0-9_]*)'

	def __init__(self, url_id='', method='0', file=None, prog=None):
		######################
		# The basic variable #
		######################
		self.id     = self.extract_id(url_id)
		self.method = method
		self.conten = {}
		################
		# The function #
		################
		self.prog = prog
		self.file = file

	def add_sub(self, sub):
		""" This function return the informations wich are write in sub.swy ."""
		sid = self.extract_id(sub)
		data = download_rss_reddit(sid)
		if data is None:
			return None
		return sid + '\t' + data['title']

	def write(self):
		self.file.add(
			url=self.content['url'],
			title=self.content['title'],
			url_uploader=self.content['url_sub'],
			image=self.content['image'],
			uploader=self.content['sub'],
			date=self.content['date'],
		)

	def real_analyzer(self):
		""" The main function  wich retrieve the informations and and write it
		in a file"""
		data = download_rss_reddit(self.id)
		if data is None:
			return
		for item in data['items']:
			self.content = self.info(item, {
				'url':     {'re':'<link href="(.+?)"'},
				'sub':     {'default': self.id},
				'url_sub': {'default':'https://www.reddit.com/' + self.id},
				'title':   {'re':'<title>(.+?)</title>'},
				'image':   {'re':'img src=&quot;(.+?)&quot;', 'default': self.NO_IMG},
				'date':    {'re':'<updated>(.+?)\+00:00</updated>', 'date':'%Y-%m-%dT%H:%M:%S'},
			})
			if self.content is not None:
				self.write()

	def old(self, url, lcl):
		""" The function wich is call with the option -o
		This function print the old channel or the dead channel."""
		pass

	def dead(self, url):
		""" The function wich is call with the option -d
		This function print the dead channel."""
		pass



from ..core.sock import download_https


def download_xml_daily(url_id, split=True):
	site = download_https('/rss/user/' + url_id, 'www.dailymotion.com')
	if site.status != '200':
		return None
	data = site.body
	if split:
		linfo = data.split('<item>')
		del linfo[0]
		if linfo == []:
			return None
		return linfo
	else:
		return data



import socket
import ssl
import re
import zlib

from .download import Download
from .tools import log

TIMEOUT = 2

def download_http(url, host='www.youtube.com'):
	site = Download(False, host)
	site.download(url)
	return site

def download_https(url, host='www.youtube.com'):
	site = Download(True, host)
	site.download(url)
	return site



import os
import sys

from .core.commands import Commands
from .core.tools import (
	log
)
from .version import __version__
from datetime import datetime


def getpath():
	if os.name == 'nt':
		path = os.path.expanduser('~') + '/.youtube_sm/'
	else:
		if os.uname().sysname == 'Linux' or os.uname().sysname == 'Darwin':
			try:
				path = os.environ['HOME'] + '/.cache/youtube_sm/'
			except KeyError:
				log.Error('HOME is not set')
				exit()
	return path

def createcache():
	path = getpath()
	if not os.path.exists(path + 'data/'):
		try:
			os.makedirs(path + 'data/')
		except:
			log.Warning('Data can\'t be create')
			exit()
	log.cache = path + 'verbose.log'
	return path

def main():
	path = createcache()
	log.Info('Hello :)')
	log.Info('Date: ', datetime.now())
	log.Info('Version: ', __version__)
	log.Info('Path: ', path)
	del sys.argv[0]
	cmd = Commands(path)
	cmd.parser()
	cmd.router()



import re
from ..core.sock import download_https


def download_xml_revolutionpermanente(split=True):
	site = download_https('/spip.php?page=backend', 'www.revolutionpermanente.fr')
	if site.status != '200':
		return None
	data = site.body
	if split:
			linfo = re.findall('<item xml:lang="fr">(.+?)</item>', data, re.DOTALL)
			if linfo == []:
					return None
			return linfo
	return data



from datetime import datetime, timedelta


def lcl_time(day=31, all_time=False):
	if all_time:
		return 0
	return int(since(day))


def since(day):
	date = datetime.now() - timedelta(days=day)
	return date.strftime("%Y%m%d000000")



from .analyzer    import Analyzer
from ..downloader.laquadrature import download_rss_quadrature
import re

class LaQuadrature_Analyzer(Analyzer):
	SITE='[laquadrature]'
	LANG_MATCH=r'fr|en|es'
	URL_MATCH=r'(?:https://|)(?:www\.|)laquadrature\.net(?:/(?P<ID>%s)|)' % LANG_MATCH
	def __init__(self, url_id='', method='0', file=None, prog=None):
		######################
		# The basic variable #
		######################
		self.id = self.extract_id(url_id)
		self.method = method
		###############################
		# Init the video informations #
		###############################
		self.content = {}
		################
		# The function #
		################
		self.prog = prog
		self.file = file

	def extract_id(self, url):
		match = re.match(self.URL_MATCH, url)
		if not match:
			if re.match(r'(%s)$' % self.LANG_MATCH, url):
				return url
			return None
		lang = match.group('ID')
		if lang is not None and re.match(r'(%s)$' % self.LANG_MATCH, lang):
			return lang
		return 'fr'

	def add_sub(self, sub):
		""" This function return the informations wich are write in sub.swy ."""
		lang = self.extract_id(sub)
		if lang is None:
			return None
		return lang + '\t' + 'la quadrature du net'

	def real_analyzer(self):
		""" The main function  wich retrieve the informations and and write it
		in a file"""
		datas = download_rss_quadrature(self.id)
		if datas is None:
			return
		for data in datas:
			content = self.info(data, {
				'url': {'re': '<link>(.+?)</link>'},
				'title': {'re': '<title>(.+?)</title>'},
				'url_uploader': {'default': 'https://www.laquadrature.net/' + self.id},
				'image': {'default': 'https://static.mediapart.fr/etmagine/default/files/media_275614/la_quadrature.png'},
				'uploader': {'default': 'La Quadrature du Net'},
				'date': {'re': '<pubDate>(.+?) \+0000</pubDate>', 'date': '%a, %d %b %Y %H:%M:%S'},
			})
			if content is not None:
				self.file.add(**content)

	def old(self, url, lcl):
		""" The function wich is call with the option -o
		This function print the old channel or the dead channel."""

	def dead(self, url):
		""" The function wich is call with the option -d
		This function print the dead channel."""



import re

from datetime import datetime
from .analyzer import Analyzer
from ..downloader.revolutionpermanente import download_xml_revolutionpermanente


class RevolutionPermanente_Analyzer(Analyzer):
	SITE='[revolutionpermanente]'
	URL_MATCH=r'(?:https://|)(?:www\.|)revolutionpermanente\.fr.*'

	def __init__(self, url_id='', method='0', file=None, prog=None):
		######################
		# The basic variable #
		######################
		self.id = url_id
		self.method = method
		###############################
		# Init the video informations #
		###############################
		self.title   = ''
		self.url     = ''
		self.url_img = ''
		self.date    = ''
		################
		# The function #
		################
		self.prog = prog
		self.file = file

	def add_sub(self, url):
		""" This function return the informations wich are write in sub.swy ."""
		return 'Revolution Permanente'

	def write(self):
		return self.file.add(
			url=self.url,
			title=self.title,
			image=self.url_img,
			uploader='Revolution Permanente',
			url_uploader='https://www.revolutionpermanente.fr/',
			date=self.date,
		)


	def real_analyzer(self):
		""" The main function wich retrieve the informations and and write it
		in a file"""
		data = download_xml_revolutionpermanente()
		if data is None:
			return
		for ele in data:
			self.content = self.info(ele, {
				'title':{'re':'<title>(.*)</title>'},
				'url':{'re':'<link>(.*)</link>'},
				'url_img':{'re':"img class='spip_logo spip_logo_right spip_logos' .* src='(.+?)'"},
				'date':{'re':'<dc:date>(.*)</dc:date>','date':'%Y-%m-%dT%H:%M:%SZ'},
			})
			if self.content is not None:
				self.write()



import re

from ..core.sock import download_https


def download_rss_reddit(sid):
	site = download_https('/' + sid + '/.rss', 'www.reddit.com')
	if site.status != '200':
		return None
	data = {}
	data['items'] = re.findall(r'<entry>(.+?)</entry>', site.body)
	title = re.findall(r'<title>(.+?)</title>', site.body)
	if title == []:
		return None
	data['title'] = title[0]
	return data



from ..core.sock import download_https


def download_html_infoconcert(gid):
	site = download_https('/artiste/{}/concerts.html'.format(gid), 'www.infoconcert.com')
	if site.status != '200':
		return None
	data = site.body
	data = data.split('id="artiste-tab-archives"')[0]
	sdat = data.split('panel panel-default date-line date-line-festival')
	del sdat[0]
	if sdat == []:
		return None
	return sdat



import re
import time
import os
import sys
import optparse

from .write import (
	Write_file,
	write_css,
	write_log
)
from .thread import (
	Run_analyze,
	old,
	dead
)
from .swy import (
	swy,
	generate_swy,
	add_sub,
	add_suburl,
	init_swy
)
from .tools import (
	del_data,
	log,
	exit_debug
)
from ..version import __version__
from ..analyzer.imports import UrlToAnalyzer


class Commands():
	def __init__(self, path):
		self.params = {
			'-a':           {'func': self._a,           'option': 'URL',    'description': "Add a sub to your sub list."},
			'-e':           {'func': self._e,           'option': '',       'description': "Edit your sub list."},
			'-h':           {'func': self._h,           'option': '',       'description': "Print this help text and exit."},
			'-l':           {'func': self._l,           'option': 'URL',    'description': "Analyze only one sub."},
			'-m':           {'func': self._m,           'option': 'MODE',   'description': "Choose the type of the output file (html, json, raw, list)."},
			'-r':           {'func': self._r,           'option': '',       'description': "Remove the cache."},
			'-t':           {'func': self._t,           'option': 'DAYS',   'description': "Select how many DAYS ago the last content written to your file will be dated ."},
			'-v':           {'func': self.nothing,      'option': '',       'description': "Verbose."},
			'--af':         {'func': self.__af,         'option': 'FILE',   'description': "Add a list of sub to your sub list."},
			'--ax':         {'func': self.__ax,         'option': 'FILE',   'description': "Add a xml file in your sub list."},
			'--cat':        {'func': self.__cat,        'option': '',       'description': "View your subscriptions."},
			'--css':        {'func': self.__css,        'option': 'STYLE',  'description': "Export the css files (light, dark, switch)."},
			'--dead':       {'func': self.__dead,       'option': '',       'description': "Show the dead channels + those who posted no videos."},
			'--help':       {'func': self._h,           'option': '',       'description': "Print this help text and exit."},
			'--html':       {'func': self.__html,       'option': '',       'description': "Recover sub with html page instead of RSS. This method recover more video."},
			'--init':       {'func': self.__init,       'option': 'FILE',   'description': "Remove all your subs and add new."},
			'--loading':    {'func': self.__loading,    'option': '',       'description': "Print a progress bar."},
			'--old':        {'func': self.__old,        'option': 'MONTHS', 'description': "Show channels who didn\'t post videos since MONTHS + dead channels."},
			'--output':     {'func': self.__output,     'option': 'FILE',   'description': "Write the output in FILE."},
			'--ultra-html': {'func': self.__ultra_html, 'option': '',       'description': "An advanced version of --html."},
			'--version':    {'func': self.__version,    'option': '',       'description': "Print version."},
		}
		self.analyze  = True
		self.count    = 7
		self.loading  = False
		self.method   = '0'
		self.mode     = 'html'
		self.output   = ''
		self.path     = path
		self.trun     = time.time()
		self.url_data = {}

	def _h(self):
		print('Usage: youtube-sm [OPTIONS]')
		print()
		print('Options:')
		for name in self.params:
				param = self.params[name]
				print('  {:12} {:8}  {}'.format(name, param['option'], param['description']))
		exit()

	def _m(self, arg):
		if arg + 1 < len(sys.argv) and sys.argv[arg + 1] in ['html', 'raw', 'list', 'view', 'json']:
			self.mode = sys.argv[arg + 1]
		else:
			exit_debug('Mode file invalid', 1)

	def _t(self, arg):
		if not re.match('(-|)[0-9]*$', sys.argv[arg + 1]):
			exit_debug('Numbers of day invalid', 1)
		self.count = int(sys.argv[arg + 1])

	def _a(self, arg):
		self.analyze = False
		if arg + 1 < len(sys.argv):
			add_suburl(sys.argv[arg+1], self.path)
		else:
			exit_debug('You Forgot An Argument (-a)', 1)

	def _e(self, arg):
		self.analyze = False
		editor = os.getenv('EDITOR')
		if editor is None:
			log.RWarning('The variable `EDITOR` is not set. `vi` is use by default')
			editor = '/bin/vi'
		os.system('{} {}sub.swy'.format(editor, self.path))

	def _l(self, arg):
		del_data(self.path)
		if arg + 1 < len(sys.argv):
			analyzer = UrlToAnalyzer(sys.argv[arg+1])
			if analyzer is None:
				log.Error('URL does not match with any site')
				exit()
			self.url_data = {analyzer.SITE: [sys.argv[arg+1]]}
		else:
			exit_debug('You forgot an argument after -l', 1)

	def _r(self, arg):
		del_data(self.path)

	def __init(self, arg):
		self.analyze = False
		init_swy(self.path, arg)

	def __af(self, arg):
		self.analyze = False
		if arg + 1 < len(sys.argv) and os.path.exists(sys.argv[arg + 1]):
			add_sub(open(sys.argv[arg + 1], 'r').read().split('\n'), self.path)
		else:
			exit_debug('File not found', 1)

	def __ax(self, arg):
		self.analyze = False
		if arg + 1 < len(sys.argv) and os.path.exists(sys.argv[arg + 1]):
			generate_swy(sys.argv[arg + 1], self.path)
		else:
			exit_debug('File not found', 1)

	def __cat(self, arg):
		self.analyze = False
		if os.path.exists(self.path + 'sub.swy'):
			with open(self.path + 'sub.swy', 'r') as file:
				while True:
					line = file.readline()
					if line == '':
						break
					print(line, end='')

	def __html(self, arg):
		self.method = '1'

	def __old(self, arg):
		self.analyze = False
		self.url_data = swy(self.path)
		log.RInfo('[*]Start of analysis')
		if re.match('[0-9]*$', sys.argv[arg + 1]):
			min_tps = int(sys.argv[arg + 1])
			old(self.url_data, min_tps)
		else:
			old(self.url_data)

	def __dead(self, arg):
		self.analyze = False
		self.url_data = swy(self.path)
		log.RInfo('Start of analysis')
		dead(self.url_data)

	def __css(self, arg):
		self.analyze = False
		try:
			os.mkdir('css')
		except:
			log.Error('CSS folder already exist or can\'t be created')
		if len(sys.argv) != arg + 1:
			write_css(sys.argv[arg+1])
		else:
			write_css('')

	def __ultra_html(self, arg):
		self.method = '2'

	def __loading(self, arg):
		self.loading = True

	def __output(self, arg):
		if arg + 1 >= len(sys.argv):
			exit_debug('You forgot an argument after --output', 1)
		self.output = sys.argv[arg+1]

	def __version(self, arg):
		log.RInfo('Version: {}'.format(__version__))

	def nothing(self, arg):
		pass

	def parser(self):
		if sys.argv == ['-h'] or sys.argv == ['--help']:
			self._h()
		else:
			for arg in range(len(sys.argv)):
				if len(sys.argv[arg]) == 0 or sys.argv[arg][0] != '-' or sys.argv[arg] == '-1':
					continue
				elif len(sys.argv[arg]) > 1:
					if sys.argv[arg] in self.params:
						self.params[sys.argv[arg]]['func'](arg)
					else:
						exit_debug("No such option: {}".format(sys.argv[arg]), 1)

	def router(self):
		if self.analyze:
			if self.url_data == {}:
				self.url_data = swy(self.path, 0)
			file = Write_file(self.output, self.path, self.mode, self.method, self.count)
			Run_analyze(self.url_data, self.loading, file, self.method)
			file.write()
			write_log(sys.argv, self.path, self.trun)
		log.Info('Done ({} seconds)'.format(str(time.time() - self.trun)[:7]))



import re

from datetime  import datetime
from .analyzer import Analyzer
from ..downloader.infoconcert import (
	download_html_infoconcert
)


class InfoConcert_Analyzer(Analyzer):
	SITE='[infoconcert]'
	URL_MATCH=r'(?:https://|)(?:www\.|)infoconcert\.com/artiste/(?P<ID>[a-z0-9-]*-[0-9]*)'

	def __init__(self, url_id='', method='0', file=None, prog=None):
		######################
		# The basic variable #
		######################
		self.id = self.extract_id(url_id)
		self.method = method
		###############################
		# Init the video informations #
		###############################
		self.channel = ''
		self.date = ''
		self.title = ''
		self.url = ''
		self.url_img = ''
		################
		# The function #
		################
		self.prog = prog
		self.file = file

	def add_sub(self, url):
		""" This function return the informations wich are write in sub.swy ."""
		sub = self.extract_id(url)
		name = re.findall('(.+?)-[0-9]*$', sub)[0].replace('-', ' ')
		return sub + '\t' + name

	def write(self):
		return self.file.add(
			url=self.url,
			title=self.title,
			image='https://www.dailydot.com/wp-content/uploads/558/89/tdd-classical_music_emojis.jpg',
			uploader=self.location,
			date=self.date,
		)

	def info_html(self, ele):
			date          = re.findall(r'<time itemprop="startDate" datetime="(.+?)"', ele)[0]
			self.date     = datetime.strptime(date, '%Y-%m-%dT%H:%M:%S')
			self.url      = 'https://www.infoconcert.com' + re.findall(r'itemprop="url"\n.*href="(.+?)"', ele)[0]
			self.location = re.findall(r'<span itemprop="name">(.+?)</span>', ele)[0]
			self.location += ' - ' + re.findall(r'<span itemprop="locality">(.+?)</span>', ele)[0]
			raw_singers   = re.findall('<div class="spectacle">(.+?)</div>', ele, re.DOTALL)[0]
			singers       = re.findall('>(.+?)</a>', raw_singers)
			self.title    = '/'.join(singers)

	def real_analyzer(self):
		""" The main function wich retrieve the informations and and write it
		in a file"""
		data = download_html_infoconcert(self.id)
		if data is None:
			return
		for ele in data:
			self.info_html(ele)
			self.write()

	def old(self, url, lcl):
		""" The function wich is call with the option -o
		This function print the old channel or the dead channel."""

	def dead(self, url):
		""" The function wich is call with the option -d
		This function print the dead channel."""



import re

from time          import strptime
from .analyzer     import Analyzer
from ..core.tools  import log
from ..downloader.dailymotion import download_xml_daily


class Dailymotion_Analyzer(Analyzer):
	SITE='[dailymotion]'
	URL_MATCH=r'(https://|)(www\.|)dailymotion\.com/'

	def __init__(self, url_id='', method='0', file=None, prog=None):
		self.id          = url_id
		self.method      = method
		# Init info videos
		self.url         = ""
		self.url_channel = ""
		self.url_img     = ""
		self.title       = ""
		self.channel     = ""
		self.date        = ""
		# Function
		self.prog = prog
		self.file = file

	def add_sub(self, sub):
		log.Warning("Dailymotion has suspended RSS. Sorry")
		return None
		data = download_xml_daily(sub, split=False)
		if data is None:
			return None
		return sub + '\t' + sub

	def real_analyzer(self):
		log.Warning("Dailymotion has suspended RSS. Sorry")
		return
		if self.method == '0':
			linfo = self._download_page()
			if linfo is False or linfo is None:
				return
			for i in linfo:
				try:
					self.info_recup_rss(i)
					self.write()
				except:
					log.Warning('Error during the retrieve of the info ({})'.format(self.id))

	def _download_page(self):
		if self.method == '0':
			return download_xml_daily(self.id)
		else:
			return False

	def write(self):
		"""Write the information in a file"""
		return self.file.add(
			uploader=self.channel,
			date=self.date,
			title=self.title,
			url=self.url,
			url_uploader=self.url_channel,
			image=self.url_img,
			view=self.view,
		)

	def info_recup_rss(self, i):
		self.url = re.findall(r'<link>(.+?)</link>', i)[0]
		self.url_channel = 'https://www.dailymotion.com/' + self.id
		self.title = re.findall(r'<media:title>(.+?)</media:title>', i)[0]
		self.url_img = re.findall(r'<media:thumbnail\ url="(.+?)"', i)[0]
		self.channel = self.id
		date = re.findall(r'<pubDate>.{5}(.*)\ \+', i)[0]
		self.date = strptime(date, '%d %b %Y %H:%M:%S')
		self.view = re.findall(r'<dm:views>(.+?)</dm:views>', i)[0].replace(',', '')

	def convert_date(self, date, y=False):
		x = strptime(date, '%d %b %Y %H:%M:%S')
		month, day = str(x.tm_mon), str(x.tm_mday)
		if len(month) == 1:
			month = '0' + month
		if len(day) == 1:
			day = '0' + day
		if y:
			return str(x.tm_year) + '-' + month + '-' + day, x
		else:
			return str(x.tm_year) + '-' + month + '-' + day

	def old(self, url, lcl):
		data = download_xml_daily(url)
		if data is None:
			print('[ channel dead ]', url)
		else:
			try:
				last = int(self.convert_date(re.findall(r'<pubDate>.{5}(.*)\ \+', data[0])[0]).replace('-', ''))
			except:
				return
			if last <= lcl:
				last = str(last)
				print('[ ' + last[:4] + '/' + last[4:6] + '/' + last[6:8] + ' ] ' + url)
		return

	def dead(self, url):
		data = download_xml_daily(url)
		if data is None or data is False:
			print('[ channel dead ]', url)



from time   import time
from .tools import log

import socket
import ssl
import re
import zlib


class Download():
	LANGUAGE = 'en'
	SETTIMEOUT = 0.1
	TIMEOUT = 3
	USERAGENT = 'Mozilla/5.0 (X11; Linux x86_64; rv:71.0) Gecko/20100101 Firefox/71.0'

	def __init__(self, https, host):
		self.ssl  = https
		self.host = host
		self.setvar()
		self.protocol = 'http'
		self.port = 80
		if self.ssl:
			self.protocol = 'https'
			self.port = 443

	def setvar(self):
		self.status   = ''
		self.headers  = ''
		self.body     = ''

	def create_request(self, path, method, body, headers):
		req = "{} {} HTTP/1.1\r\n".format(method, path)

		default = {
			'Host': self.host,
			'User-Agent': self.USERAGENT,
			'Accept': '*/*',
			'Accept-Encoding': 'gzip',
			'Accept-Language': self.LANGUAGE
		}
		for key in headers:
			default[key] = headers[key]
		if body is not None:
			default["Content-Length"] = str(len(body))
		for name in default:
			req += '{}: {}\r\n'.format(name, default[name])
		req += "\r\n"
		if body is not None:
			req += body
		return req.encode()

	def real_recv(self):
		try:
			raw_data = self.sock.recv(100000)
			return raw_data
		except socket.timeout:
			return b''
		except Exception as e:
			log.Error('Error Recv: {}'.format(e))
			return b''

	def recv_headers(self):
		data = b''
		trun = time()
		while True:
			data += self.real_recv()
			if b'\r\n\r\n' in data:
				self.headers = data.split(b'\r\n\r\n')[0].decode()
				len_headers = len(self.headers) + 4
				self.status = re.findall(r'HTTP/1\.1 ([0-9]*)', self.headers)[0]
				data = data[len_headers:]
				if self.status != '100':
					return data
				trun = time()
			elif time() - trun > self.TIMEOUT:
				return None

	def recv_body(self, data, dlen):
		trun = time()
		while True:
			data += self.real_recv()
			if (dlen == -1 and time() - trun > self.TIMEOUT) \
				or (dlen != -1 and dlen <= len(data)):
				return data

	def recv_chunk_body(self, chunk):
		dlen = 0
		data = b''
		chunk = b'\r\n' + chunk
		while True:
			while dlen + 2 < len(chunk) and b'\r\n' in chunk[dlen + 2:]:
				data += chunk[:dlen]
				chunk = chunk[dlen + 2:]
				str_dlen = chunk.split(b'\r\n')[0].decode()
				dlen = int(str_dlen, 16)
				if dlen == 0:
					return data
				chunk = chunk[len(str_dlen) + 2:]
			chunk += self.real_recv()

	def recv(self):
		body = self.recv_headers()
		if self.headers == '' or self.status == '204' or body is None:
			return
		if re.match('.*[Cc]ontent-[Ll]ength:', self.headers, re.DOTALL):
			dlen = int(re.findall('[Cc]ontent-[Ll]ength: (.*)\r', self.headers)[0])
			body = self.recv_body(body, dlen)
		elif re.match('.*[Tt]ransfer-[Ee]ncoding:[ \t]*[Cc]hunked', self.headers, re.DOTALL):
			body = self.recv_chunk_body(body)
		else:
			body = self.recv_body(body, -1)

		try:
			if re.match('.*[Cc]ontent-[Ee]ncoding:[ \t]*gzip', self.headers, re.DOTALL):
				self.body = zlib.decompress(body, 16+zlib.MAX_WBITS).decode()
			else:
				self.body = body.decode('utf8')
		except Exception as e:
			log.Error(e)

	def connect(self):
		self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
		if self.ssl:
			context = ssl._create_default_https_context()
			context.check_hostname = self.host
			self.sock = context.wrap_socket(self.sock, server_hostname=self.host)
		self.sock.connect((self.host, self.port))

	def download(self, path, method="get", headers={}, body="", status='200'):
		trun = time()
		self.setvar()
		method = method.upper()

		request = self.create_request(path, method, body, headers)
		try:
			self.connect()
		except Exception as e:
			log.Error(str(e))
			return
		self.sock.send(request)
		self.sock.settimeout(self.SETTIMEOUT)
		self.recv()

		url = "{}://{}{}".format(self.protocol, self.host, path)
		if self.status == status:
			log.Info("{} {} {} {} {}".format(method, url, self.status, len(self.body), time() - trun))
		else:
			log.Warning("{} {} {} {} {}".format(method, url, self.status, len(self.body), time() - trun))



from ..core.tools import log
from .dailymotion import Dailymotion_Analyzer
from .infoconcert import InfoConcert_Analyzer
from .laquadrature import LaQuadrature_Analyzer
from .peertube    import Peertube_Analyzer
from .reddit      import Reddit_Analyzer
from .revolutionpermanente import RevolutionPermanente_Analyzer
from .youtube     import Youtube_Analyzer

analyzers = [
	Dailymotion_Analyzer,
	InfoConcert_Analyzer,
	Peertube_Analyzer,
	Reddit_Analyzer,
	RevolutionPermanente_Analyzer,
	Youtube_Analyzer,
	LaQuadrature_Analyzer,
]


def return_Analyzer(site):
	for anal in analyzers:
		if anal.SITE == site:
			return anal
	log.Error('Analyzer not found ({})'.format(site))
	return None


def UrlToAnalyzer(url):
	for anal in analyzers:
		if anal().match(url):
			return anal
	return None



import re
import socket

from datetime     import datetime
from .analyzer    import Analyzer
from ..core.tools import log
from ..downloader.peertube import download_xml_peertube


class Peertube_Analyzer(Analyzer):
	SITE='[peertube]'
	INSTANCES=r'91video\.online|fightforinfo\.com|video\.lewd\.host|alttube\.fr|aperi\.tube|armstube\.com|artitube\.artifaille\.fr|atchao\.info|banneddata\.me|betamax\.video|bittube\.video|cattube\.org|cinema\.yunohost\.support|conf\.tube|csictv\.csic\.es|devtube\.dev-wiki\.de|dialup\.express|diode\.zone|diytelevision\.com|doby\.io|dreiecksnebel\.alex-detsch\.de|elo-country\.patault\.ovh|evertron\.tv|exode\.me|fanvid\.stopthatimp\.net|fightforinfo\.com|flix\.librenet\.co\.za|fontube\.fr|framatube\.org|freespeech\.tube|gouttedeau\.space|greatview\.video|highvoltage\.tv|hitchtube\.fr|hostyour\.tv|indymotion\.fr|infotik\.fr|irrsinn\.video|juggling\.digital|lepetitmayennais\.fr\.nf|lexx\.impa\.me|libre\.tube|libre\.video|libretube\.net|lostpod\.space|luxtube\.lu|manicphase\.me|marudpeertube\.damnserver\.com|media\.assassinate-you\.net|media\.krashboyz\.org|media\.privacyinternational\.org|media\.zat\.im|medias\.libox\.fr|medias\.pingbase\.net|megatube\.lilomoino\.fr|merci-la-police\.fr|monplaisirtube\.ddns\.net|mplayer\.demouliere\.eu|mytape\.org|mytube\.madzel\.de|nuage\.acostey\.fr|open\.tube|p\.eertu\.be|p0\.pm|peer\.hostux\.social|peer\.mathdacloud\.ovh|peer\.philoxweb\.be|peer\.tube|peertube\.020\.pl|peertube\.1312\.media|peertube\.alcalyn\.app|peertube\.alpharius\.io|peertube\.alter-nativ-voll\.de|peertube\.amicale\.net|peertube\.anarchmusicall\.net|peertube\.anduin\.net|peertube\.anon-kenkai\.com|peertube\.anzui\.de|peertube\.artica\.center|peertube\.askan\.info|peertube\.asrun\.eu|peertube\.at|peertube\.bierzilla\.fr|peertube\.bilange\.ca|peertube\.bittube\.tv|peertube\.ch|peertube\.chantierlibre\.org|peertube\.cipherbliss\.com|peertube\.club|peertube\.co\.uk|peertube\.cojo\.uno|peertube\.cpy\.re|peertube\.cyber-tribal\.com|peertube\.datagueule\.tv|peertube\.david\.durieux\.family|peertube\.ddns\.net|peertube\.debian\.social|peertube\.demonix\.fr|peertube\.desmu\.fr|peertube\.devosi\.org|peertube\.dk|peertube\.donnadieu\.fr|peertube\.dsmouse\.net|peertube\.dynlinux\.io|peertube\.education-forum\.com|peertube\.eric\.ovh|peertube\.esadhar\.net|peertube\.euskarabildua\.eus|peertube\.f-si\.org|peertube\.fedi\.quebec|peertube\.fedilab\.app|peertube\.ffs2play\.fr|peertube\.floss-marketing-school\.com|peertube\.fr|peertube\.freeforge\.eu|peertube\.gaialabs\.ch|peertube\.gcaillaut\.fr|peertube\.gcfamily\.fr|peertube\.gegeweb\.eu|peertube\.gwendalavir\.eu|peertube\.hatthieves\.es|peertube\.heberge\.fr|peertube\.heraut\.eu|peertube\.hugolecourt\.fr|peertube\.iriseden\.eu|peertube\.iselfhost\.com|peertube\.jackbot\.fr|peertube\.kangoulya\.org|peertube\.koehn\.com|peertube\.kosebamse\.com|peertube\.la-famille-muller\.fr|peertube\.laas\.fr|peertube\.lagob\.fr|peertube\.librelabucm\.org|peertube\.linuxrocks\.online|peertube\.live|peertube\.livingutopia\.org|peertube\.lol|peertube\.love|peertube\.mablr\.org|peertube\.makotoworkshop\.org|peertube\.mckillop\.org|peertube\.me|peertube\.montecsys\.fr|peertube\.musicstudio\.pro|peertube\.mxinfo\.fr|peertube\.mygaia\.org|peertube\.nayya\.org|peertube\.nexon\.su|peertube\.nocturlab\.fr|peertube\.nomagic\.uk|peertube\.normandie-libre\.fr|peertube\.opencloud\.lu|peertube\.openstreetmap\.fr|peertube\.opentunisia\.org|peertube\.parleur\.net|peertube\.pcservice46\.fr|peertube\.pl|peertube\.pontostroy\.gq|peertube\.qtg\.fr|peertube\.quaylessed\.icu|peertube\.rainbowswingers\.net|peertube\.roflcopter\.fr|peertube\.schaeferit\.de|peertube\.servebeer\.com|peertube\.serveur\.slv-valbonne\.fr|peertube\.simounet\.net|peertube\.sl-network\.fr|peertube\.slat\.org|peertube\.snargol\.com|peertube\.social\.my-wan\.de|peertube\.solidev\.net|peertube\.stemy\.me|peertube\.storais\.org|peertube\.su|peertube\.swarm\.solvingmaz\.es|peertube\.tamanoir\.foucry\.net|peertube\.tech|peertube\.terranout\.mine\.nu|peertube\.the-penguin\.de|peertube\.ti-fr\.com|peertube\.togart\.de|peertube\.touhoppai\.moe|peertube\.tronic-studio\.com|peertube\.umeahackerspace\.se|peertube\.underworld\.fr|peertube\.uno|peertube\.ventresmous\.fr|peertube\.video|peertube\.volaras\.net|peertube\.we-keys\.fr|peertube\.xoddark\.com|peertube\.xtenz\.xyz|peertube\.zapashcanon\.fr|peertube\.zergy\.net|peertube2\.020\.pl|peertube2\.cpy\.re|peertube3\.cpy\.re|peervideo\.club|peervideo\.ru|peerwatch\.xyz|pire\.artisanlogiciel\.net|pony\.tube|porntube\.ddns\.net|ppstube\.portageps\.org|pt\.forty-two\.nl|pt\.kircheneuenburg\.de|pt\.laurentkruger\.fr|pt\.neko\.bar|pt\.pube\.tk|ptube\.horsentiers\.fr|ptube\.rousset\.nom\.fr|pytu\.be|raptube\.antipub\.org|refuznik\.video|replay\.jres\.org|repro\.video|runtube\.re|scitech\.video|share\.tube|skeptikon\.fr|srv1\.gerifilmai\.ynh\.fr|stage\.peertube\.ch|superduper\.space|thickrips\.cloud|thinkerview\.video|toobnix\.org|troll\.tv|tube\.22decembre\.eu|tube\.4aem\.com|tube\.ac-amiens\.fr|tube\.ac-lyon\.fr|tube\.anjara\.eu|tube\.aps\.systems|tube\.azbyka\.ru|tube\.backbord\.net|tube\.benzo\.online|tube\.blob\.cat|tube\.bn4t\.me|tube\.bootlicker\.party|tube\.bruniau\.net|tube\.calculate\.social|tube\.conferences-gesticulees\.net|tube\.crapaud-fou\.org|tube\.danq\.me|tube\.delalande\.me|tube\.dragonpsi\.xyz|tube\.egf\.mn|tube\.extinctionrebellion\.fr|tube\.fab-l3\.org|tube\.fabrigli\.fr|tube\.fdn\.fr|tube\.fede\.re|tube\.govital\.net|tube\.grin\.hu|tube\.h3z\.jp|tube\.hoga\.fr|tube\.homecomputing\.fr|tube\.interhacker\.space|tube\.kagouille\.fr|tube\.kapussinettes\.ovh|tube\.kdy\.ch|tube\.kher\.nl|tube\.kicou\.info|tube\.ksl-bmx\.de|tube\.lain\.church|tube\.lesamarien\.fr|tube\.linc\.systems|tube\.lou\.lt|tube\.maiti\.info|tube\.maliweb\.at|tube\.midov\.pl|tube\.minzord\.eu\.org|tube\.misterbanal\.net|tube\.mochi\.academy|tube\.nah\.re|tube\.nemsia\.org|tube\.netzspielplatz\.de|tube\.nox-rhea\.org|tube\.nuagelibre\.fr|tube\.nx-pod\.de|tube\.nx12\.net|tube\.odat\.xyz|tube\.openalgeria\.org|tube\.otter\.sh|tube\.ouahpiti\.info|tube\.p2p\.legal|tube\.pasa\.tf|tube\.piweb\.be|tube\.plaf\.fr|tube\.plus200\.com|tube\.postblue\.info|tube\.rebellion\.global|tube\.rfc1149\.net|tube\.rita\.moe|tube\.svnet\.fr|tube\.taker\.fr|tube\.tappret\.fr|tube\.tchncs\.de|tube\.tesgo\.fr|tube\.thechangebook\.org|tube\.traydent\.info|tube\.troopers\.agency|tube\.undernet\.uy|tube\.valinor\.fr|tube\.worldofhauru\.xyz|tube\.yukimochi\.jp|tv\.datamol\.org|tv\.derdorifer\.org|tv\.lapesto\.fr|tv\.mooh\.fr|us\.tv|v\.kretschmann\.social|v\.lastorder\.xyz|v\.lesterpig\.com|v\.mbius\.io|vault\.mle\.party|vid\.lelux\.fi|vid\.lubar\.me|vid\.ncrypt\.at|vid\.y-y\.li|vidcommons\.org|video\.1000i100\.fr|video\.amic37\.fr|video\.anormallostpod\.ovh|video\.antirep\.net|video\.antopie\.org|video\.autizmo\.xyz|video\.blender\.org|video\.blueline\.mg|video\.bruitbruit\.com|video\.cabane-libre\.org|video\.colibris-outilslibres\.org|video\.connor\.money|video\.conquerworld\.fr|video\.coop\.tools|video\.deadsuperhero\.com|video\.devinberg\.com|video\.die-partei\.social|video\.emergeheart\.info|video\.farci\.org|video\.fdlibre\.eu|video\.fediverso\.net|video\.fitchfamily\.org|video\.g3l\.org|video\.gcfam\.net|video\.glassbeadcollective\.org|video\.greenmycity\.eu|video\.gresille\.org|video\.hackers\.town|video\.hardlimit\.com|video\.hdys\.band|video\.ihatebeinga\.live|video\.imagotv\.fr|video\.iphodase\.fr|video\.irem\.univ-paris-diderot\.fr|video\.isurf\.ca|video\.lacaveatonton\.ovh|video\.latavernedejohnjohn\.fr|video\.lemediatv\.fr|video\.lequerrec\.eu|video\.lewd\.host|video\.liberta\.vip|video\.linc\.systems|video\.livecchi\.cloud|video\.lono\.space|video\.lqdn\.fr|video\.lw1\.at|video\.mantlepro\.com|video\.marcorennmaus\.de|video\.migennes\.net|video\.monarch-pass\.net|video\.monsieurbidouille\.fr|video\.nesven\.eu|video\.netsyms\.com|video\.oh14\.de|video\.okaris\.de|video\.omniatv\.com|video\.passageenseine\.fr|video\.ploud\.fr|video\.ploud\.jp|video\.qoto\.org|video\.rastapuls\.com|video\.selea\.se|video\.sftblw\.moe|video\.splat\.soy|video\.subak\.ovh|video\.taboulisme\.com|video\.tedomum\.net|video\.turbo\.chat|video\.typica\.us|video\.valme\.io|video\.vny\.fr|video\.wivodaim\.net|video\.writeas\.org|video\.yukari\.moe|videomensoif\.ynh\.fr|videonaute\.fr|videos-libr\.es|videos\.adhocmusic\.com|videos\.alolise\.org|videos\.benpro\.fr|videos\.cemea\.org|videos\.cloudfrancois\.fr|videos\.coletivos\.org|videos\.dinofly\.com|videos\.domainepublic\.net|videos\.ensilib\.re|videos\.festivalparminous\.org|videos\.funkwhale\.audio|videos\.gerdemann\.me|videos\.globenet\.org|videos\.govanify\.com|videos\.hack2g2\.fr|videos\.hauspie\.fr|videos\.iut-orsay\.fr|videos\.koumoul\.com|videos\.koweb\.fr|videos\.lavoixdessansvoix\.org|videos\.lescommuns\.org|videos\.numerique-en-commun\.fr|videos\.pair2jeux\.tube|videos\.pofilo\.fr|videos\.pueseso\.club|videos\.side-ways\.net|videos\.squat\.net|videos\.tcit\.fr|videos\.ubuntu-paris\.org|videos\.uni-corn\.me|videos\.upr\.fr|videos\.wakapo\.com|videotape\.me|vidz\.dou\.bet|viid\.ga|vis\.ion\.ovh|vloggers\.social|vod\.ksite\.de|vod\.mochi\.academy|watch\.44con\.com|watch\.snoot\.tube|wetube\.ojamajo\.moe|widemus\.de|www\.videos-libr\.es|www\.yiny\.org|xxxporn\.co\.uk|yt\.is\.nota\.live|yunopeertube\.myddns\.me'
	#'
	URL_MATCH=r'(?:http[s]?://|)(?P<Host>%s)/feeds/videos\.xml\?accountId=(?P<ID>[0-9]*)' % INSTANCES

	def __init__(self, url_id='', method='0', file=None, prog=None):
		######################
		# The basic variable #
		######################
		self.id      = self.extract_sub(url_id)
		self.method  = method
		###############################
		# Init the video informations #
		###############################
		self.channel     = ''
		self.date        = ''
		self.title       = ''
		self.url         = ''
		self.url_channel = ''
		self.url_img     = ''
		################
		# The function #
		################
		self.prog = prog
		self.file = file

	def extract_sub(self, url):
		match = self.match(url)
		if match:
			return '{}:{}'.format(match.group('Host'), match.group('ID'))
		return url

	def add_sub(self, url):
		""" This function return the informations wich are write in sub.swy ."""
		sub = self.extract_sub(url)
		try:
			data = download_xml_peertube(sub, False)
		except socket.gaierror:
			log.Error("Wrong host ({})".format(sub))
			return None
		if data is None or '</rss>' not in data:
			log.Error("This sub can't be add.")
			return None
		title = re.findall(r'<title>(.+?)</title>', data)
		if title != []:
			return sub + '\t' + title[0]
		log.Error("This sub can't be add.")

	def _download_page(self):
		return download_xml_peertube(self.id)

	def write(self):
		"""Write the information in a file"""
		return self.file.add(
			uploader=self.channel,
			date=self.date,
			title=self.title,
			url=self.url,
			url_uploader=self.url_channel,
			image=self.url_img,
		)

	def real_analyzer(self):
		""" The main function  wich retrieve the informations and and write it
		in a file"""
		linfo = self._download_page()
		if linfo is False or linfo is None:
			return
		for i in linfo:
			try:
				self.info_rss(i)
				self.write()
			except:
				log.Error('Error during the retrieve of the info ({})'.format(self.id))

	def info_rss(self, i):
		self.url         = re.findall(r'<link>(.*)</link>', i)[0]
		self.url_channel = 'https://' + self.id.split(':')[0]
		self.channel     = re.findall(r'<dc:creator>(.*)</dc:creator>', i)[0]
		self.title       = re.findall(r'<title><!\[CDATA\[(.*)\]\]></title>', i)[0]
		self.url_img     = re.findall(r'<media:thumbnail url="(.*)"', i)[0]
		date             = re.findall(r'<pubDate>(.*)</pubDate>', i)[0]
		self.date        = datetime.strptime(date, '%a, %d %b %Y %H:%M:%S GMT')

	def old(self, url, lcl):
		""" The function wich is call with the option -o
		This function print the old channel or the dead channel."""
		pass

	def dead(self, url):
		""" The function wich is call with the option -d
		This function print the dead channel."""
		pass



from setuptools	import setup, find_packages
from os		import path

here = path.abspath(path.dirname(__file__))
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
	README = f.read()

exec(compile(open('youtube_sm/version.py').read(),
             'youtube_sm/version.py', 'exec'))

setup(
	name='youtube_sm',
	version=__version__,
	url='https://github.com/sawyerf/youtube-sm',
	author='Sawyerf',
	author_email='sawyer.flink@protonmail.ch',
	description='Youtube subscription manager',
	long_description_content_type='text/markdown',
	long_description=README,
	keywords='youtube subscription api manager html',
	license='MLP-2.0',
	packages=find_packages(include=[
		'youtube_sm', 'youtube_sm.*',
	]),
	classifiers=[
		'Topic :: Internet :: WWW/HTTP :: Indexing/Search',
		'Environment :: Console',
		'License :: OSI Approved :: Mozilla Public License 2.0 (MPL 2.0)',
		'Operating System :: Microsoft :: Windows',
		'Operating System :: POSIX :: Linux',
		'Programming Language :: Python',
		'Programming Language :: Python :: 3',
		'Programming Language :: Python :: 3 :: Only'
	],
	entry_points={
		'console_scripts': ['youtube-sm = youtube_sm.main:main']
	}
)



import re
from datetime  import datetime
from threading import Thread


class Analyzer(Thread):
	"""
	Common Function to all Analyzers
	"""
	NO_IMG='https://sawyerf.gitlab.io/youtube_sm/pixel.png'
	def Thread(self):
		Thread.__init__(self)

	def run(self):
		self.real_analyzer()
		if self.prog is not None:
			self.prog.add()

	def match(self, url):
		return re.match(self.URL_MATCH, url)

	def extract_id(self, url):
		if url == '':
			return ''
		match = self.match(url)
		if not match:
			return url
		return match.group('ID')

	def info(self, item, matchs):
		content = {}
		for var, match in matchs.items():
			if 're' not in match:
				if 'default' in match:
					content[var] = match['default']
					continue
				else:
					return None
			find = re.findall(match['re'], item)
			i = 0
			if 'i' in match:
				i = match['i']
			if len(find) <= i:
				if 'default' not in match:
					return None
				content[var] = match['default']
			else:
				content[var] = find[i]
				if 'date' in match:
					try:
						content[var] = datetime.strptime(content[var], match['date'])
					except:
						return None
		return content



from ..core.sock import download_https
from ..core.tools import log


def ptube_crtlink(info, type_file):
	if len(info) == 2 and info[1] != '':
		return '/feeds/videos.{}?accountId={}'.format(type_file, info[1])
	return '/feeds/videos.{}'.format(type_file)


def download_xml_peertube(url_id, split=True):
	info = url_id.split(':')
	site = download_https(ptube_crtlink(info, 'xml'), info[0])
	if site.status != '200':
		site = download_https(ptube_crtlink(info, 'xml').replace('accountId', 'videoChannelId'), info[0])
		if site.status != '200':
			log.Error('Failed to download ({})'.format(info))
			return None
	data = site.body
	if split:
		linfo = data.split('<item>')
		del linfo[0]
		if linfo == []:
			return None
		return linfo
	else:
		return data



import time
import os

from datetime import datetime, timedelta
import json


class Write_file():
	def __init__(self, output='sub.html', path_cache='', mode='html', method='0', since=7):
		if output != '':
			self.output = output
		else:
			self.output = {
				'html': 'sub.html',
				'json': 'sub.json',
				'list': 'sub_list',
				'raw':  'sub_raw',
			}[mode]
		self.file = open(self.output, 'w')
		self.mode = mode
		self.method = method
		self.contents = []
		self.path_cache = path_cache
		self.since = since
		self.data_path = f'{self.path_cache}data/contents.{self.method}.json'
		if os.path.exists(self.data_path):
			with open(self.data_path, 'r') as f:
				self.contents = json.load(f)
				f.close()
			for content in self.contents:
				if content['date'] != '':
					content['date'] = datetime.strptime(content['date'], '%Y-%m-%d %H:%M:%S')

	def add(self, url='', title='', url_uploader='', image='', uploader='', date='', view=''):
		date = date.replace(microsecond=0)
		for content in self.contents:
			if content['url']['content'] == url:
				return
		content = {
			'url': {
				'content': url,
				'image': image,
				'uploader': url_uploader,
			},
			'date': date,
			'title': title,
			'uploader': uploader,
			'views': view,
		}
		self.contents.append(content)

	def write(self):
		router = {
			'html': self.html,
			'raw': self.raw,
			'list': self.list,
			'json': self.json,
		}
		router[self.mode]()
		self.file.close()
		with open(self.data_path, 'w') as f:
			json.dump(self.contents, f, indent='\t', default=str)
			f.close()

	def RangeContent(self):
		self.contents = sorted(self.contents, key=lambda k: k['date'], reverse=True)
		if self.since == -1:
			return self.contents
		now = datetime.now() + timedelta(days=3)
		since = datetime.now() - timedelta(days=self.since)
		ncontents = []
		for content in self.contents:
			if since < content['date'] and content['date'] < now:
				ncontents.append(content)
		return ncontents

	def html(self):
		self.file.write(
			'<html>\n' + \
			'	<head>\n' + \
			'		<meta charset="utf-8" />\n' + \
			'		<link rel="stylesheet" href="css/sub.css" />\n' + \
			'		<link rel="stylesheet" media="screen and (max-width: 1081px)" href="css/sub_mobile.css"/>\n' + \
			'		<title>Abonnements</title>\n' + \
			'	</head>\n' + \
			'	<body>\n' + \
			'		<!-- {} -->\n'.format(time.ctime()) + \
			'		<div class="But"><input value="" id="but" type="submit" onclick="lol()"></div>\n' + \
			'		<div id="first"></div>\n' + \
			'		<script type="text/javascript" src="css/button.js"></script>\n'
		)
		for content in self.RangeContent():
			self.file.write(
				f'		<!--NEXT -->\n' + \
				f'		<div class="video">\n' + \
				f'			<a class="left" href="{content["url"]["content"]}">\n' + \
				f'				<div class="container">\n' + \
				f'					<img src="{content["url"]["image"]}">\n' + \
				f'					<div class="bottom-right"></div>\n' + \
				f'				</div>\n' + \
				f'			</a>\n' + \
				f'			<a href="{content["url"]["content"]}"><h4>{content["title"]}</h4> </a>\n' + \
				f'			<a href="{content["url"]["uploader"]}"> <p>{content["uploader"]}</p> </a>\n' + \
				f'			<p>{content["date"]}</p>\n' + \
				f'			<p class="clear"></p>\n' + \
				f'		</div>\n' + \
				f'		\n'
			)
		self.file.write('</body></html>')

	def json(self):
		"""
		Write json file
		"""
		contents = self.RangeContent()
		json.dump(contents, self.file, indent='\t', default=str)
		return True

	def raw(self):
		"""
		Write raw file
		"""
		for content in self.RangeContent():
			self.file.write('{date}	{url[content]}	{url[uploader]}	{title}	{uploader}	{url[image]}\n'.format(**content))
		return True

	def list(self):
		"""
		Write list file
		"""
		for content in self.RangeContent():
			self.file.write('{url[content]}\n'.format(**content))


def write_css(arg):
	if arg == '' or arg == 'light' or arg[0] == '-':
		open('css/sub.css', 'w').write(':root {\n	--Back: white;\n	--DivHover : #e9edf3;\n	--Div: white;\n	--ColorH4: black;\n	--ColorP: grey;\n	--ColorBut: #e2e5e9;\n 	--HeightBut: 0px;\n	--HeightButMob: 0px }\n')
	elif arg == 'dark':
		open('css/sub.css', 'w').write(':root {\n	--Back: #181a1d;\n	--DivHover : #1c1e21;\n	--Div: #26292e;\n	--ColorH4: #dcdddf;\n	--ColorP: grey;\n	--ColorBut: #e2e5e9;\n	--HeightBut: 0px;\n	--HeightButMob: 0px }\n')
	elif arg == 'switch':
		open('css/button.js', 'w').write('function lol() {\n	if (lel == "1") {\n		document.documentElement.style.setProperty("--Back", "#181a1d");\n		document.documentElement.style.setProperty("--Div", "#1c1e21");\n		document.documentElement.style.setProperty("--DivHover", "#26292e");\n		document.documentElement.style.setProperty("--ColorH4", "#dcdddf");\n		document.documentElement.style.setProperty("--ColorP", "grey");\n		document.documentElement.style.setProperty("--ColorBut", "#292c30");\n		document.cookie = "lel=1";\n		lel = "2"\n	} else if (lel == "2"){\n		document.documentElement.style.setProperty("--Back", "white");\n		document.documentElement.style.setProperty("--Div", "white");\n		document.documentElement.style.setProperty("--DivHover", "white");\n		document.documentElement.style.setProperty("--ColorH4", "black");\n		document.documentElement.style.setProperty("--ColorP", "grey");\n		document.documentElement.style.setProperty("--ColorBut", "#e2e5e9");\n		document.cookie = "lel=2";\n		lel = "1"} }\ndocument.documentElement.style.setProperty("--HeightBut", "25px");\ndocument.documentElement.style.setProperty("--HeightButMob", "50px");\nif (document.cookie == ""){	var lel = "1"\n} else { lel = document.cookie.replace("lel=", "")}\nlol();')
		open('css/sub.css', 'w').write(':root {\n	--Back: white;\n	--DivHover : #e9edf3;\n	--Div: white;\n	--ColorH4: black;\n	--ColorP: grey;\n	--ColorBut: #e2e5e9;\n 	--HeightBut: 0px;\n	--HeightButMob: 0px }\n')
	else:
		exit('[!] No such argument: ' + arg)
	open('css/sub.css', 'a').write(':root {\n--Back: #181a1d;\n--DivHover : #1c1e21;\n--Div: #26292e;\n--ColorH4: #dcdddf;\n--ColorP: grey;\n--ColorBut: #e2e5e9;\n--HeightBut: 0px;\n--HeightButMob: 0px }\n.left { float: left; }\n.clear { clear: both; }\n* { font-family: Arial;}\n\nbody {\nmargin: 0 0 0 0;\nbackground-color: var(--Back);}\n\n.But {\nwidth: 100%;\nposition: fixed;}\n\n#but {\n-webkit-appearance: none;\nborder: none;\nbackground-color: var(--ColorBut);\nwidth: 100%;\nheight: var(--HeightBut); }\n\n#first{\nmargin: 0 27% 0 27%;\nheight: var(--HeightBut); }\n\ndiv.video:hover{ \nbackground-color: var(--DivHover); }\n\ndiv.video {\nbackground-color: var(--Div);\nmargin: 5px 27% 5px 27%; }\n\nimg {\nwidth: 280px;\nheight: 157px;\nmargin-right: 5px;\nobject-fit: cover;\n}\n\nh4 {\ncolor: var(--ColorH4);\nline-height: 18px;\nfont-size: 18px;\nmargin: 0px 0px 10px 0px; }\n\np {\ncolor: var(--ColorP);\nmargin: 4px 0px 4px 0px;\n}\n\na{\ntext-decoration: none;\ncolor: black; }\n\n.description {\noverflow: hidden;\ndisplay: -webkit-box;\n-webkit-line-clamp: 3;\n-webkit-box-orient: vertical;\n  white-space: pre;\n}\n.container {\nposition: relative;\ntext-align: center;\ncolor: white;\n}\n\n/* Bottom right text */\n.bottom-right {\nposition: absolute;\nbackground-color: rgba(25, 25, 25, 0.5);\npadding: 5 5 5 5;\nmargin-right: 5;\nbottom: 0%;\nright: 0%;\n}\n')
	open('css/sub_mobile.css', 'w').write('.left { float:left; }\n.clear { clear: both; }\n* { font-family: Arial; }\n\n\n.description {\noverflow: hidden;\ndisplay: -webkit-box;\n-webkit-line-clamp: 1;\n-webkit-box-orient: vertical;\n}\n#But{\nmargin: 0 0 0 0;\nposition: fixed;\nwidth: 100%;\n}\n\n#but {\nheight: var(--HeightButMob);\nmargin: 0 0 0 0;}\n\n#first{\nmargin: 0 0 0 0;\nwidth: 100%;\nheight: var(--HeightButMob);\n}\n\ndiv.video {\nmargin-left: 0%;\nmargin-right: 0%;\nmargin: 10px 10px 10px 10px; }\n\n\nimg {\nwidth: 380px;\nheight: 214px;\nobject-fit: cover;\n}\n\nh4 {\noverflow: hidden;\ndisplay: -webkit-box;\n-webkit-line-clamp: 3;\n-webkit-box-orient: vertical;\nline-height: 30px;\nfont-size: 30px;\nmargin: 0px 0px 0px 0px;\n}\n\np {\ncolor: grey;\n#line-height: 5px;\nfont-size: 1.9em;\nmargin: 0 0 0 0;\n}\n\na {\ntext-decoration: none;\ncolor: black;\n}\n\n.container {\nposition: relative;\ntext-align: center;\ncolor: white;\n}\n\n/* Bottom right text */\n.bottom-right {\nposition: absolute;\nbackground-color: rgba(25, 25, 25, 0.5);\npadding: 5 5 5 5;\nmargin-right: 5;\nbottom: 0%;\nright: 0%;\n}\n')


def write_log(arg, path, passe):
	var = ''
	for i in arg:
		var += i + ' '
	open(path + 'log', 'a', encoding='utf8').write(time.strftime("%H%M") + '\t' + str(time.time() - passe)[:4] + '\t' + var + '\n')



from ..core.sock import download_https
import re

def download_rss_quadrature(lang):
	url = '/%s/feed/' % lang
	if lang == 'fr':
		url = '/feed/'
	site = download_https(url, 'www.laquadrature.net')
	if site.status != '200':
		return None
	data = re.findall('<item>(.+?)</item>', site.body, re.DOTALL)
	return data



import os
import sys
from shutil import rmtree
from time import time, strftime, gmtime

TRUN=time()


class Progress():
	"""Print a progress bar"""
	def __init__(self, xmax=0):
		self.xmin = 0
		self.xmax = xmax

	def add(self):
		self.xmin += 1
		self.progress_bar()

	def progress_bar(self):
		load = ''
		if self.xmax == 0:
			pc = 0
		else:
			pc = (self.xmin/self.xmax)
		for i in range(int(pc*40)):
			load += '█'
		for i in range(int(40 - pc * 40 + (pc * 40) % 1)):
			load += ' '
		print('{} %|{}| {}/{} analyzed'.format(str(pc*100)[:3], load, str(self.xmin), str(self.xmax)), end='\r')
		if pc == 1:
			print()


class Progress_loop():
	def __init__(self):
		self.x = 0
		self.bar = '          '
		self.xmin = None
		self.xmax = None

	def add(self):
		self.x += 1
		self.progress_bar()

	def progress_bar(self):
		self.x = self.x % 10
		print('|{}{}{}|'.format(self.bar[:self.x], '█', self.bar[self.x + 1:]), end='\r')


def del_data(path):
	"""
	delete the data folder
	"""
	if os.path.exists(path + 'data'):
		try:
			rmtree(path + 'data')
		except:
			pass
		os.makedirs(path + 'data/')
	else:
		log.Warning('Data folder don\'t exist')


def exit_debug(msg, i=0):
	if i == 0:
		log.RInfo(msg)
	elif i == 1:
		log.Error(msg)
	exit()


class log:
	cache=''

	def Info(*msg, end='\n'):
		log.print('\33[1;36m', '[*] ', msg, end, True)

	def Warning(*msg, end='\n'):
		log.print('\33[1;33m', '[!] ', msg, end, True)

	def RInfo(*msg, end='\n'):
		log.print('\33[1;36m', '[*] ', msg, end, False)

	def RWarning(*msg, end='\n'):
		log.print('\33[1;33m', '[!] ', msg, end, False)

	def Error(*msg, end='\n'):
		log.print('\33[1;31m', '[!] ', msg, end, False)

	def Join(msgs):
		fin = ''
		for msg in msgs:
			fin += str(msg)
		return fin

	def print(color, init, msgs, end, verbose_only):
		date = '[{}]'.format(strftime("%H:%M:%S", gmtime(time() - TRUN)))
		msg = log.Join(msgs)
		if (verbose_only and '-v' in sys.argv) or not verbose_only:
			print(color, init, msg, "\033[00m", end=end, sep='')
		if log.cache != '':
			open(log.cache, 'a').write(date + init + str(msg) + '\n')



from threading import Thread
from ..analyzer.imports import return_Analyzer
from .time import lcl_time
from .tools import (
	Progress,
	Progress_loop,
	log)


def Run_analyze(urls, loading=False, file=None, method='0'):
	"""Run all the analyze in a thread"""
	threads = []
	nb = 0
	for site in urls:
		nb += len(urls[site])
	if loading and nb == 1:
		prog = Progress_loop()
		prog.progress_bar()
	elif loading and nb > 1:
		prog = Progress(nb)
		prog.progress_bar()
	else:
		prog = None
	if method == '0':
		max_thr = 40
	elif method == '1':
		max_thr = 50
	elif method == '2':
		max_thr = 5
	# Run threads
	log.Info('Start threads')
	for site in urls:
		analyzer = return_Analyzer(site)
		if analyzer is None:
			continue
		for i in range(len(urls[site])):
			thr = analyzer(urls[site][i], method, file, prog)
			thr.Thread()
			threads.append(thr)
			log.Info("Thread start ({}{})".format(site, urls[site][i]))
			thr.start()
			if i > 1 and (i % max_thr == 0 or nb == i+1):
				for y in threads:
					y.join()
				threads = []
		for y in threads:
			y.join()
	if loading and prog.xmin != prog.xmax:
		prog.xmin = prog.xmax
		prog.progress_bar()
		print()


def old(subs, min_tps=12):
	lcl = lcl_time(min_tps * 31)
	threads = []
	log.Info('Start threads')
	for site in subs:
		analyzer = return_Analyzer(site)()
		for url in subs[site]:
			thr = Thread(target=analyzer.old, args=(url, lcl))
			threads.append(thr)
			thr.start()
	for i in threads:
		i.join()


def dead(subs):
	threads = []
	log.Info('Start threads')
	for site in subs:
		analyzer = return_Analyzer(site)()
		for url in subs[site]:
			thr = Thread(target=analyzer.dead, args=(url,))
			threads.append(thr)
			thr.start()
	for i in threads:
		i.join()
